from zope.interface import implements
from zope.schema import Object

from plone.namedfile.interfaces import INamedFileField, INamedImageField
from plone.namedfile.interfaces import INamedBlobFileField, INamedBlobImageField

from plone.namedfile.interfaces import INamedFile, INamedImage
from plone.namedfile.interfaces import INamedBlobFile, INamedBlobImage

from plone.namedfile.file import NamedFile as FileValueType
from plone.namedfile.file import NamedImage as ImageValueType
from plone.namedfile.file import NamedBlobFile as BlobFileValueType
from plone.namedfile.file import NamedBlobImage as BlobImageValueType

class NamedFile(Object):
    """A NamedFile field
    """
    implements(INamedFileField)
    
    _type = FileValueType
    schema = INamedFile
    
    def __init__(self, **kw):
        if 'schema' in kw:
            self.schema = kw.pop('schema')
        super(NamedFile, self).__init__(schema=self.schema, **kw)

class NamedImage(Object):
    """A NamedImage field
    """
    implements(INamedImageField)
    
    _type = ImageValueType
    schema = INamedImage
    
    def __init__(self, **kw):
        if 'schema' in kw:
            self.schema = kw.pop('schema')
        super(NamedImage, self).__init__(schema=self.schema, **kw)

class NamedBlobFile(Object):
    """A NamedBlobFile field
    """
    implements(INamedBlobFileField)
    
    _type = BlobFileValueType
    schema = INamedBlobFile
    
    def __init__(self, **kw):
        if 'schema' in kw:
            self.schema = kw.pop('schema')
        super(NamedBlobFile, self).__init__(schema=self.schema, **kw)

class NamedBlobImage(Object):
    """A NamedBlobImage field
    """
    implements(INamedBlobImageField)
    
    _type = BlobImageValueType
    schema = INamedBlobImage
    
    def __init__(self, **kw):
        if 'schema' in kw:
            self.schema = kw.pop('schema')
        super(NamedBlobImage, self).__init__(schema=self.schema, **kw)


from zope.interface import implements

from zope.app.file.file import File
from zope.app.file.image import Image

from z3c.blobfile.file import File as BlobFile
from z3c.blobfile.image import Image as BlobImage

from plone.namedfile.interfaces import INamedFile, INamedImage
from plone.namedfile.interfaces import INamedBlobFile, INamedBlobImage
from plone.namedfile.utils import get_contenttype

class NamedFile(File):
    """A non-BLOB file that stores a filename
    """
    implements(INamedFile)

    def __init__(self, data='', contentType='', filename=None):
        if filename is not None and contentType in ('', 'application/octet-stream'):
            contentType = get_contenttype(filename=filename)
        super(NamedFile, self).__init__(data, contentType)
        self.filename = filename

class NamedImage(Image):
    """An non-BLOB image with a filename
    """
    implements(INamedImage)

    def __init__(self, data='', contentType='', filename=None):
        super(NamedImage, self).__init__(data)
        self.filename = filename
        
        # Allow override of the image sniffer
        if contentType:
            self.contentType = contentType

class NamedBlobFile(BlobFile):
    """A file stored in a ZODB BLOB, file a filename
    """
    implements(INamedBlobFile)

    def __init__(self, data='', contentType='', filename=None):
        if filename is not None and contentType in ('', 'application/octet-stream'):
            contentType = get_contenttype(filename=filename)
        super(NamedBlobFile, self).__init__(data, contentType)
        self.filename = filename

class NamedBlobImage(BlobImage):
    """An image stored in a ZODB BLOB with a filename
    """
    implements(INamedBlobImage)

    def __init__(self, data='', contentType='', filename=None):
        super(NamedBlobImage, self).__init__(data)
        self.filename = filename
        
        # Allow override of the image sniffer
        if contentType:
            self.contentType = contentType


from zope.interface import Interface

from zope import schema
from zope.schema.interfaces import IObject

from zope.app.file.interfaces import IFile, IImage
from z3c.blobfile.interfaces import IBlobFile, IBlobImage

# File objects

class INamed(Interface):
    """An item with a filename
    """
    
    filename = schema.TextLine(title=u"Filename", required=False, default=None)

class IBlobby(Interface):
    """Marker interface for objects that support blobs.
    """

class INamedFile(INamed, IFile):
    """A non-BLOB file with a filename
    """
    
class INamedBlobFile(INamedFile, IBlobby, IBlobFile):
    """A BLOB file with a filename
    """

class INamedImage(INamed, IImage):
    """A non-BLOB image with a filename
    """

class INamedBlobImage(INamedImage, IBlobby, IBlobImage):
    """A BLOB image with a filename
    """

# Fields

class INamedField(IObject):
    """Base field type
    """

class INamedFileField(INamedField):
    """Field for storing INamedFile objects.
    """

class INamedBlobFileField(INamedFileField):
    """Field for storing INamedBlobFile objects.
    """

class INamedImageField(INamedField):
    """Field for storing INamedImage objects.
    """

class INamedBlobImageField(INamedImageField):
    """Field for storing INamedBlobImage objects.
    """